@@ -12,7 +12,6 @@ import matplotlib.pyplot as plt
 
 from . import utils
 from .palettes import color_palette, blend_palette
-from .external.six import string_types
 from .distributions import distplot, kdeplot,  _freedman_diaconis_bins
 
 
@@ -43,7 +42,8 @@ class Grid(object):
         Parameters
         ----------
         legend_data : dict, optional
-            Dictionary mapping label names to matplotlib artist handles. The
+            Dictionary mapping label names (or two-element tuples where the
+            second element is a label name) to matplotlib artist handles. The
             default reads from ``self._legend_data``.
         title : string, optional
             Title for the legend. The default reads from ``self._hue_var``.
@@ -61,7 +61,8 @@ class Grid(object):
 
         """
         # Find the data for the legend
-        legend_data = self._legend_data if legend_data is None else legend_data
+        if legend_data is None:
+            legend_data = self._legend_data
         if label_order is None:
             if self.hue_names is None:
                 label_order = list(legend_data.keys())
@@ -76,16 +77,26 @@ class Grid(object):
         except TypeError:  # labelsize is something like "large"
             title_size = mpl.rcParams["axes.labelsize"]
 
+        # Unpack nested labels from a hierarchical legend
+        labels = []
+        for entry in label_order:
+            if isinstance(entry, tuple):
+                _, label = entry
+            else:
+                label = entry
+            labels.append(label)
+
         # Set default legend kwargs
         kwargs.setdefault("scatterpoints", 1)
 
         if self._legend_out:
 
             kwargs.setdefault("frameon", False)
+            kwargs.setdefault("loc", "center right")
 
             # Draw a full-figure legend outside the grid
-            figlegend = self.fig.legend(handles, label_order, "center right",
-                                        **kwargs)
+            figlegend = self.fig.legend(handles, labels, **kwargs)
+
             self._legend = figlegend
             figlegend.set_title(title, prop={"size": title_size})
 
@@ -115,7 +126,9 @@ class Grid(object):
         else:
             # Draw a legend in the first axis
             ax = self.axes.flat[0]
-            leg = ax.legend(handles, label_order, loc="best", **kwargs)
+            kwargs.setdefault("loc", "best")
+
+            leg = ax.legend(handles, labels, **kwargs)
             leg.set_title(title, prop={"size": title_size})
 
         return self
@@ -225,7 +238,7 @@ class FacetGrid(Grid):
         # Handle deprecations
         if size is not None:
             height = size
-            msg = ("The `size` paramter has been renamed to `height`; "
+            msg = ("The `size` parameter has been renamed to `height`; "
                    "please update your code.")
             warnings.warn(msg, UserWarning)
 
@@ -384,8 +397,8 @@ class FacetGrid(Grid):
         The plots it produces are often called "lattice", "trellis", or
         "small-multiple" graphics.
 
-        It can also represent levels of a third varaible with the ``hue``
-        parameter, which plots different subets of data in different colors.
+        It can also represent levels of a third variable with the ``hue``
+        parameter, which plots different subsets of data in different colors.
         This uses color to resolve elements on a third dimension, but only
         draws subsets on top of each other and will not tailor the ``hue``
         parameter for the specific visualization the way that axes-level
@@ -568,7 +581,7 @@ class FacetGrid(Grid):
             >>> def qqplot(x, y, **kwargs):
             ...     _, xr = stats.probplot(x, fit=False)
             ...     _, yr = stats.probplot(y, fit=False)
-            ...     plt.scatter(xr, yr, **kwargs)
+            ...     sns.scatterplot(xr, yr, **kwargs)
             >>> g = sns.FacetGrid(tips, col="smoker", hue="sex")
             >>> g = (g.map(qqplot, "total_bill", "tip", **kws)
             ...       .add_legend())
@@ -892,23 +905,27 @@ class FacetGrid(Grid):
         return self
 
     def set_xticklabels(self, labels=None, step=None, **kwargs):
-        """Set x axis tick labels on the bottom row of the grid."""
+        """Set x axis tick labels of the grid."""
         for ax in self.axes.flat:
             if labels is None:
-                labels = [l.get_text() for l in ax.get_xticklabels()]
+                curr_labels = [l.get_text() for l in ax.get_xticklabels()]
                 if step is not None:
                     xticks = ax.get_xticks()[::step]
-                    labels = labels[::step]
+                    curr_labels = curr_labels[::step]
                     ax.set_xticks(xticks)
-            ax.set_xticklabels(labels, **kwargs)
+                ax.set_xticklabels(curr_labels, **kwargs)
+            else:
+                ax.set_xticklabels(labels, **kwargs)
         return self
 
     def set_yticklabels(self, labels=None, **kwargs):
         """Set y axis tick labels on the left column of the grid."""
         for ax in self.axes.flat:
             if labels is None:
-                labels = [l.get_text() for l in ax.get_yticklabels()]
-            ax.set_yticklabels(labels, **kwargs)
+                curr_labels = [l.get_text() for l in ax.get_yticklabels()]
+                ax.set_yticklabels(curr_labels, **kwargs)
+            else:
+                ax.set_yticklabels(labels, **kwargs)
         return self
 
     def set_titles(self, template=None, row_template=None,  col_template=None,
@@ -1085,7 +1102,7 @@ class PairGrid(Grid):
     the marginal distribution of each variable can be shown on the diagonal.
 
     It can also represent an additional level of conditionalization with the
-    ``hue`` parameter, which plots different subets of data in different
+    ``hue`` parameter, which plots different subsets of data in different
     colors. This uses color to resolve elements on a third dimension, but
     only draws subsets on top of each other and will not tailor the ``hue``
     parameter for the specific visualization the way that axes-level functions
@@ -1097,8 +1114,8 @@ class PairGrid(Grid):
 
     def __init__(self, data, hue=None, hue_order=None, palette=None,
                  hue_kws=None, vars=None, x_vars=None, y_vars=None,
-                 diag_sharey=True, height=2.5, aspect=1,
-                 despine=True, dropna=True, size=None):
+                 corner=False, diag_sharey=True, height=2.5, aspect=1,
+                 layout_pad=0, despine=True, dropna=True, size=None):
         """Initialize the plot figure and PairGrid object.
 
         Parameters
@@ -1107,7 +1124,8 @@ class PairGrid(Grid):
             Tidy (long-form) dataframe where each column is a variable and
             each row is an observation.
         hue : string (variable name), optional
-            Variable in ``data`` to map plot aspects to different colors.
+            Variable in ``data`` to map plot aspects to different colors. This
+            variable will be excluded from the default x and y variables.
         hue_order : list of strings
             Order for the levels of the hue variable in the palette
         palette : dict or seaborn color palette
@@ -1123,10 +1141,15 @@ class PairGrid(Grid):
         {x, y}_vars : lists of variable names, optional
             Variables within ``data`` to use separately for the rows and
             columns of the figure; i.e. to make a non-square plot.
+        corner : bool, optional
+            If True, don't add axes to the upper (off-diagonal) triangle of the
+            grid, making this a "corner" plot.
         height : scalar, optional
             Height (in inches) of each facet.
         aspect : scalar, optional
             Aspect * height gives the width (in inches) of each facet.
+        layout_pad : scalar, optional
+            Padding between axes; passed to ``fig.tight_layout``.
         despine : boolean, optional
             Remove the top and right spines from the plots.
         dropna : boolean, optional
@@ -1217,9 +1240,9 @@ class PairGrid(Grid):
             :context: close-figs
 
             >>> g = sns.PairGrid(iris)
-            >>> g = g.map_upper(plt.scatter)
-            >>> g = g.map_lower(sns.kdeplot, cmap="Blues_d")
-            >>> g = g.map_diag(sns.kdeplot, lw=3, legend=False)
+            >>> g = g.map_upper(sns.scatterplot)
+            >>> g = g.map_lower(sns.kdeplot, colors="C0")
+            >>> g = g.map_diag(sns.kdeplot, lw=2)
 
         Use different colors and markers for each categorical level:
 
@@ -1228,7 +1251,7 @@ class PairGrid(Grid):
 
             >>> g = sns.PairGrid(iris, hue="species", palette="Set2",
             ...                  hue_kws={"marker": ["o", "s", "D"]})
-            >>> g = g.map(plt.scatter, linewidths=1, edgecolor="w", s=40)
+            >>> g = g.map(sns.scatterplot, linewidths=1, edgecolor="w", s=40)
             >>> g = g.add_legend()
 
         """
@@ -1236,7 +1259,7 @@ class PairGrid(Grid):
         # Handle deprecations
         if size is not None:
             height = size
-            msg = ("The `size` paramter has been renamed to `height`; "
+            msg = ("The `size` parameter has been renamed to `height`; "
                    "please update your code.")
             warnings.warn(UserWarning(msg))
 
@@ -1249,6 +1272,8 @@ class PairGrid(Grid):
                 raise ValueError("Must specify `x_vars` and `y_vars`")
         else:
             numeric_cols = self._find_numeric_cols(data)
+            if hue in numeric_cols:
+                numeric_cols.remove(hue)
             x_vars = numeric_cols
             y_vars = numeric_cols
 
@@ -1269,14 +1294,32 @@ class PairGrid(Grid):
                                  sharex="col", sharey="row",
                                  squeeze=False)
 
+        # Possibly remove upper axes to make a corner grid
+        # Note: setting up the axes is usually the most time-intensive part
+        # of using the PairGrid. We are foregoing the speed improvement that
+        # we would get by just not setting up the hidden axes so that we can
+        # avoid implementing plt.subplots ourselves. But worth thinking about.
+        self._corner = corner
+        if corner:
+            hide_indices = np.triu_indices_from(axes, 1)
+            for i, j in zip(*hide_indices):
+                try:
+                    axes[i, j].remove()
+                except NotImplementedError:  # Problem on old matplotlibs?
+                    axes[i, j].set_axis_off()
+                axes[i, j] = None
+
         self.fig = fig
         self.axes = axes
         self.data = data
 
         # Save what we are going to do with the diagonal
         self.diag_sharey = diag_sharey
+        self.diag_vars = None
         self.diag_axes = None
 
+        self._dropna = dropna
+
         # Label the axes
         self._add_axis_labels()
 
@@ -1302,8 +1345,9 @@ class PairGrid(Grid):
 
         # Make the plot look nice
         if despine:
+            self._despine = True
             utils.despine(fig=fig)
-        fig.tight_layout()
+        fig.tight_layout(pad=layout_pad)
 
     def map(self, func, **kwargs):
         """Plot with the same function in every subplot.
@@ -1316,92 +1360,9 @@ class PairGrid(Grid):
             called ``color`` and  ``label``.
 
         """
-        kw_color = kwargs.pop("color", None)
-        for i, y_var in enumerate(self.y_vars):
-            for j, x_var in enumerate(self.x_vars):
-                hue_grouped = self.data.groupby(self.hue_vals)
-                for k, label_k in enumerate(self.hue_names):
-
-                    # Attempt to get data for this level, allowing for empty
-                    try:
-                        data_k = hue_grouped.get_group(label_k)
-                    except KeyError:
-                        data_k = pd.DataFrame(columns=self.data.columns,
-                                              dtype=np.float)
-
-                    ax = self.axes[i, j]
-                    plt.sca(ax)
-
-                    # Insert the other hue aesthetics if appropriate
-                    for kw, val_list in self.hue_kws.items():
-                        kwargs[kw] = val_list[k]
-
-                    color = self.palette[k] if kw_color is None else kw_color
-                    func(data_k[x_var], data_k[y_var],
-                         label=label_k, color=color, **kwargs)
-
-                self._clean_axis(ax)
-                self._update_legend_data(ax)
-
-        if kw_color is not None:
-            kwargs["color"] = kw_color
-        self._add_axis_labels()
-
-        return self
-
-    def map_diag(self, func, **kwargs):
-        """Plot with a univariate function on each diagonal subplot.
-
-        Parameters
-        ----------
-        func : callable plotting function
-            Must take an x array as a positional argument and draw onto the
-            "currently active" matplotlib Axes. Also needs to accept kwargs
-            called ``color`` and  ``label``.
-
-        """
-        # Add special diagonal axes for the univariate plot
-        if self.square_grid and self.diag_axes is None:
-            diag_axes = []
-            for i, (var, ax) in enumerate(zip(self.x_vars,
-                                              np.diag(self.axes))):
-                if i and self.diag_sharey:
-                    diag_ax = ax._make_twin_axes(sharex=ax,
-                                                 sharey=diag_axes[0],
-                                                 frameon=False)
-                else:
-                    diag_ax = ax._make_twin_axes(sharex=ax, frameon=False)
-                diag_ax.set_axis_off()
-                diag_axes.append(diag_ax)
-            self.diag_axes = np.array(diag_axes, np.object)
-
-        # Plot on each of the diagonal axes
-        fixed_color = kwargs.pop("color", None)
-        for i, var in enumerate(self.x_vars):
-            ax = self.diag_axes[i]
-            hue_grouped = self.data[var].groupby(self.hue_vals)
-
-            plt.sca(ax)
-
-            for k, label_k in enumerate(self.hue_names):
-
-                # Attempt to get data for this level, allowing for empty
-                try:
-                    data_k = np.asarray(hue_grouped.get_group(label_k))
-                except KeyError:
-                    data_k = np.array([])
-
-                if fixed_color is None:
-                    color = self.palette[k]
-                else:
-                    color = fixed_color
-
-                func(data_k, label=label_k, color=color, **kwargs)
-
-            self._clean_axis(ax)
-
-        self._add_axis_labels()
-
+        row_indices, col_indices = np.indices(self.axes.shape)
+        indices = zip(row_indices.flat, col_indices.flat)
+        self._map_bivariate(func, indices, **kwargs)
         return self
 
     def map_lower(self, func, **kwargs):
@@ -1415,39 +1376,8 @@ class PairGrid(Grid):
             called ``color`` and  ``label``.
 
         """
-        kw_color = kwargs.pop("color", None)
-        for i, j in zip(*np.tril_indices_from(self.axes, -1)):
-            hue_grouped = self.data.groupby(self.hue_vals)
-            for k, label_k in enumerate(self.hue_names):
-
-                # Attempt to get data for this level, allowing for empty
-                try:
-                    data_k = hue_grouped.get_group(label_k)
-                except KeyError:
-                    data_k = pd.DataFrame(columns=self.data.columns,
-                                          dtype=np.float)
-
-                ax = self.axes[i, j]
-                plt.sca(ax)
-
-                x_var = self.x_vars[j]
-                y_var = self.y_vars[i]
-
-                # Insert the other hue aesthetics if appropriate
-                for kw, val_list in self.hue_kws.items():
-                    kwargs[kw] = val_list[k]
-
-                color = self.palette[k] if kw_color is None else kw_color
-                func(data_k[x_var], data_k[y_var], label=label_k,
-                     color=color, **kwargs)
-
-            self._clean_axis(ax)
-            self._update_legend_data(ax)
-
-        if kw_color is not None:
-            kwargs["color"] = kw_color
-        self._add_axis_labels()
-
+        indices = zip(*np.tril_indices_from(self.axes, -1))
+        self._map_bivariate(func, indices, **kwargs)
         return self
 
     def map_upper(self, func, **kwargs):
@@ -1461,40 +1391,8 @@ class PairGrid(Grid):
             called ``color`` and  ``label``.
 
         """
-        kw_color = kwargs.pop("color", None)
-        for i, j in zip(*np.triu_indices_from(self.axes, 1)):
-
-            hue_grouped = self.data.groupby(self.hue_vals)
-
-            for k, label_k in enumerate(self.hue_names):
-
-                # Attempt to get data for this level, allowing for empty
-                try:
-                    data_k = hue_grouped.get_group(label_k)
-                except KeyError:
-                    data_k = pd.DataFrame(columns=self.data.columns,
-                                          dtype=np.float)
-
-                ax = self.axes[i, j]
-                plt.sca(ax)
-
-                x_var = self.x_vars[j]
-                y_var = self.y_vars[i]
-
-                # Insert the other hue aesthetics if appropriate
-                for kw, val_list in self.hue_kws.items():
-                    kwargs[kw] = val_list[k]
-
-                color = self.palette[k] if kw_color is None else kw_color
-                func(data_k[x_var], data_k[y_var], label=label_k,
-                     color=color, **kwargs)
-
-            self._clean_axis(ax)
-            self._update_legend_data(ax)
-
-        if kw_color is not None:
-            kwargs["color"] = kw_color
-
+        indices = zip(*np.triu_indices_from(self.axes, 1))
+        self._map_bivariate(func, indices, **kwargs)
         return self
 
     def map_offdiag(self, func, **kwargs):
@@ -1510,15 +1408,144 @@ class PairGrid(Grid):
         """
 
         self.map_lower(func, **kwargs)
-        self.map_upper(func, **kwargs)
+        if not self._corner:
+            self.map_upper(func, **kwargs)
         return self
 
+    def map_diag(self, func, **kwargs):
+        """Plot with a univariate function on each diagonal subplot.
+
+        Parameters
+        ----------
+        func : callable plotting function
+            Must take an x array as a positional argument and draw onto the
+            "currently active" matplotlib Axes. Also needs to accept kwargs
+            called ``color`` and  ``label``.
+
+        """
+        # Add special diagonal axes for the univariate plot
+        if self.diag_axes is None:
+            diag_vars = []
+            diag_axes = []
+            for i, y_var in enumerate(self.y_vars):
+                for j, x_var in enumerate(self.x_vars):
+                    if x_var == y_var:
+
+                        # Make the density axes
+                        diag_vars.append(x_var)
+                        ax = self.axes[i, j]
+                        diag_ax = ax.twinx()
+                        diag_ax.set_axis_off()
+                        diag_axes.append(diag_ax)
+
+                        # Work around matplotlib bug
+                        # https://github.com/matplotlib/matplotlib/issues/15188
+                        if not plt.rcParams.get("ytick.left", True):
+                            for tick in ax.yaxis.majorTicks:
+                                tick.tick1line.set_visible(False)
+
+                        # Remove main y axis from density axes in a corner plot
+                        if self._corner:
+                            ax.yaxis.set_visible(False)
+                            if self._despine:
+                                utils.despine(ax=ax, left=True)
+                            # TODO add optional density ticks (on the right)
+                            # when drawing a corner plot?
+
+            if self.diag_sharey:
+                # This may change in future matplotlibs
+                # See https://github.com/matplotlib/matplotlib/pull/9923
+                group = diag_axes[0].get_shared_y_axes()
+                for ax in diag_axes[1:]:
+                    group.join(ax, diag_axes[0])
+
+            self.diag_vars = np.array(diag_vars, np.object)
+            self.diag_axes = np.array(diag_axes, np.object)
+
+        # Plot on each of the diagonal axes
+        fixed_color = kwargs.pop("color", None)
+
+        for var, ax in zip(self.diag_vars, self.diag_axes):
+            hue_grouped = self.data[var].groupby(self.hue_vals)
+
+            plt.sca(ax)
+
+            for k, label_k in enumerate(self.hue_names):
+
+                # Attempt to get data for this level, allowing for empty
+                try:
+                    # TODO newer matplotlib(?) doesn't need array for hist
+                    data_k = np.asarray(hue_grouped.get_group(label_k))
+                except KeyError:
+                    data_k = np.array([])
+
+                if fixed_color is None:
+                    color = self.palette[k]
+                else:
+                    color = fixed_color
+
+                if self._dropna:
+                    data_k = utils.remove_na(data_k)
+
+                func(data_k, label=label_k, color=color, **kwargs)
+
+            self._clean_axis(ax)
+
+        self._add_axis_labels()
+
+        return self
+
+    def _map_bivariate(self, func, indices, **kwargs):
+        """Draw a bivariate plot on the indicated axes."""
+        kws = kwargs.copy()  # Use copy as we insert other kwargs
+        kw_color = kws.pop("color", None)
+        for i, j in indices:
+            x_var = self.x_vars[j]
+            y_var = self.y_vars[i]
+            ax = self.axes[i, j]
+            self._plot_bivariate(x_var, y_var, ax, func, kw_color, **kws)
+        self._add_axis_labels()
+
+    def _plot_bivariate(self, x_var, y_var, ax, func, kw_color, **kwargs):
+        """Draw a bivariate plot on the specified axes."""
+        plt.sca(ax)
+        if x_var == y_var:
+            axes_vars = [x_var]
+        else:
+            axes_vars = [x_var, y_var]
+        hue_grouped = self.data.groupby(self.hue_vals)
+        for k, label_k in enumerate(self.hue_names):
+
+            # Attempt to get data for this level, allowing for empty
+            try:
+                data_k = hue_grouped.get_group(label_k)
+            except KeyError:
+                data_k = pd.DataFrame(columns=axes_vars,
+                                      dtype=np.float)
+
+            if self._dropna:
+                data_k = data_k[axes_vars].dropna()
+
+            x = data_k[x_var]
+            y = data_k[y_var]
+
+            for kw, val_list in self.hue_kws.items():
+                kwargs[kw] = val_list[k]
+            color = self.palette[k] if kw_color is None else kw_color
+
+            func(x, y, label=label_k, color=color, **kwargs)
+
+        self._clean_axis(ax)
+        self._update_legend_data(ax)
+
     def _add_axis_labels(self):
         """Add labels to the left and bottom Axes."""
         for ax, label in zip(self.axes[-1, :], self.x_vars):
             ax.set_xlabel(label)
         for ax, label in zip(self.axes[:, 0], self.y_vars):
             ax.set_ylabel(label)
+        if self._corner:
+            self.axes[0, 0].set_ylabel("")
 
     def _find_numeric_cols(self, data):
         """Find which variables in a DataFrame are numeric."""
@@ -1591,7 +1618,7 @@ class JointGrid(object):
 
             >>> import matplotlib.pyplot as plt
             >>> g = sns.JointGrid(x="total_bill", y="tip", data=tips)
-            >>> g = g.plot_joint(plt.scatter, color=".5", edgecolor="white")
+            >>> g = g.plot_joint(sns.scatterplot, color=".5")
             >>> g = g.plot_marginals(sns.distplot, kde=False, color=".5")
 
         Draw the two marginal plots separately:
@@ -1601,39 +1628,13 @@ class JointGrid(object):
 
             >>> import numpy as np
             >>> g = sns.JointGrid(x="total_bill", y="tip", data=tips)
-            >>> g = g.plot_joint(plt.scatter, color="m", edgecolor="white")
+            >>> g = g.plot_joint(sns.scatterplot, color="m")
             >>> _ = g.ax_marg_x.hist(tips["total_bill"], color="b", alpha=.6,
             ...                      bins=np.arange(0, 60, 5))
             >>> _ = g.ax_marg_y.hist(tips["tip"], color="r", alpha=.6,
             ...                      orientation="horizontal",
             ...                      bins=np.arange(0, 12, 1))
 
-        Add an annotation with a statistic summarizing the bivariate
-        relationship:
-
-        .. plot::
-            :context: close-figs
-
-            >>> from scipy import stats
-            >>> g = sns.JointGrid(x="total_bill", y="tip", data=tips)
-            >>> g = g.plot_joint(plt.scatter,
-            ...                  color="g", s=40, edgecolor="white")
-            >>> g = g.plot_marginals(sns.distplot, kde=False, color="g")
-            >>> g = g.annotate(stats.pearsonr)
-
-        Use a custom function and formatting for the annotation
-
-        .. plot::
-            :context: close-figs
-
-            >>> g = sns.JointGrid(x="total_bill", y="tip", data=tips)
-            >>> g = g.plot_joint(plt.scatter,
-            ...                  color="g", s=40, edgecolor="white")
-            >>> g = g.plot_marginals(sns.distplot, kde=False, color="g")
-            >>> rsquare = lambda a, b: stats.pearsonr(a, b)[0] ** 2
-            >>> g = g.annotate(rsquare, template="{stat}: {val:.2f}",
-            ...                stat="$R^2$", loc="upper left", fontsize=12)
-
         Remove the space between the joint and marginal axes:
 
         .. plot::
@@ -1668,7 +1669,7 @@ class JointGrid(object):
         if size is not None:
             height = size
             msg = ("The `size` parameter has been renamed to `height`; "
-                   "pleaes update your code.")
+                   "please update your code.")
             warnings.warn(msg, UserWarning)
 
         # Set up the subplot grid
@@ -1704,7 +1705,7 @@ class JointGrid(object):
             y = data.get(y, y)
 
         for var in [x, y]:
-            if isinstance(var, string_types):
+            if isinstance(var, str):
                 err = "Could not interpret input '{}'".format(var)
                 raise ValueError(err)
 
@@ -1909,11 +1910,11 @@ class JointGrid(object):
 def pairplot(data, hue=None, hue_order=None, palette=None,
              vars=None, x_vars=None, y_vars=None,
              kind="scatter", diag_kind="auto", markers=None,
-             height=2.5, aspect=1, dropna=True,
+             height=2.5, aspect=1, corner=False, dropna=True,
              plot_kws=None, diag_kws=None, grid_kws=None, size=None):
     """Plot pairwise relationships in a dataset.
 
-    By default, this function will create a grid of Axes such that each
+    By default, this function will create a grid of Axes such that each numeric
     variable in ``data`` will by shared in the y-axis across a single row and
     in the x-axis across a single column. The diagonal Axes are treated
     differently, drawing a plot to show the univariate distribution of the data
@@ -1946,7 +1947,7 @@ def pairplot(data, hue=None, hue_order=None, palette=None,
         columns of the figure; i.e. to make a non-square plot.
     kind : {'scatter', 'reg'}, optional
         Kind of plot for the non-identity relationships.
-    diag_kind : {'auto', 'hist', 'kde'}, optional
+    diag_kind : {'auto', 'hist', 'kde', None}, optional
         Kind of plot for the diagonal subplots. The default depends on whether
         ``"hue"`` is used or not.
     markers : single matplotlib marker code or list, optional
@@ -1958,15 +1959,21 @@ def pairplot(data, hue=None, hue_order=None, palette=None,
         Height (in inches) of each facet.
     aspect : scalar, optional
         Aspect * height gives the width (in inches) of each facet.
+    corner : bool, optional
+        If True, don't add axes to the upper (off-diagonal) triangle of the
+        grid, making this a "corner" plot.
     dropna : boolean, optional
         Drop missing values from the data before plotting.
     {plot, diag, grid}_kws : dicts, optional
-        Dictionaries of keyword arguments.
+        Dictionaries of keyword arguments. ``plot_kws`` are passed to the
+        bivariate plotting function, ``diag_kws`` are passed to the univariate
+        plotting function, and ``grid_kws`` are passed to the :class:`PairGrid`
+        constructor.
 
     Returns
     -------
-    grid : PairGrid
-        Returns the underlying ``PairGrid`` instance for further tweaking.
+    grid : :class:`PairGrid`
+        Returns the underlying :class:`PairGrid` instance for further tweaking.
 
     See Also
     --------
@@ -2032,6 +2039,13 @@ def pairplot(data, hue=None, hue_order=None, palette=None,
         ...                  x_vars=["sepal_width", "sepal_length"],
         ...                  y_vars=["petal_width", "petal_length"])
 
+    Plot only the lower triangle of bivariate axes:
+
+    .. plot::
+        :context: close-figs
+
+        >>> g = sns.pairplot(iris, corner=True)
+
     Use kernel density estimates for univariate plots:
 
     .. plot::
@@ -2061,7 +2075,7 @@ def pairplot(data, hue=None, hue_order=None, palette=None,
     if size is not None:
         height = size
         msg = ("The `size` parameter has been renamed to `height`; "
-               "pleaes update your code.")
+               "please update your code.")
         warnings.warn(msg, UserWarning)
 
     if not isinstance(data, pd.DataFrame):
@@ -2069,18 +2083,14 @@ def pairplot(data, hue=None, hue_order=None, palette=None,
             "'data' must be pandas DataFrame object, not: {typefound}".format(
                 typefound=type(data)))
 
-    if plot_kws is None:
-        plot_kws = {}
-    if diag_kws is None:
-        diag_kws = {}
-    if grid_kws is None:
-        grid_kws = {}
+    plot_kws = {} if plot_kws is None else plot_kws.copy()
+    diag_kws = {} if diag_kws is None else diag_kws.copy()
+    grid_kws = {} if grid_kws is None else grid_kws.copy()
 
     # Set up the PairGrid
-    diag_sharey = diag_kind == "hist"
+    grid_kws.setdefault("diag_sharey", diag_kind == "hist")
     grid = PairGrid(data, vars=vars, x_vars=x_vars, y_vars=y_vars, hue=hue,
-                    hue_order=hue_order, palette=palette,
-                    diag_sharey=diag_sharey,
+                    hue_order=hue_order, palette=palette, corner=corner,
                     height=height, aspect=aspect, dropna=dropna, **grid_kws)
 
     # Add the markers here as PairGrid has figured out how many levels of the
@@ -2257,18 +2267,15 @@ def jointplot(x, y, data=None, kind="scatter", stat_func=None,
     # Handle deprecations
     if "size" in kwargs:
         height = kwargs.pop("size")
-        msg = ("The `size` paramter has been renamed to `height`; "
+        msg = ("The `size` parameter has been renamed to `height`; "
                "please update your code.")
         warnings.warn(msg, UserWarning)
 
     # Set up empty default kwarg dicts
-    if joint_kws is None:
-        joint_kws = {}
+    joint_kws = {} if joint_kws is None else joint_kws.copy()
     joint_kws.update(kwargs)
-    if marginal_kws is None:
-        marginal_kws = {}
-    if annot_kws is None:
-        annot_kws = {}
+    marginal_kws = {} if marginal_kws is None else marginal_kws.copy()
+    annot_kws = {} if annot_kws is None else annot_kws.copy()
 
     # Make a colormap based off the plot color
     if color is None:
